explanation_types = {'Explicación del modelo': 1,
                     'Explicación de la variable': 2,
                     'Explicación de la muestra': 3,
                     'What If': 4,
                     'What If Battle': 5}